/*
	FltRangeCmd.cc

	Will Eighteen, 2000: will@opaline.demon.co.uk
*/

#include <stdio.h>
#include "FltRangeCmd.h"
#include "Application.h"
#include <LineEdit.h>
#include <InfoDialogManager.h>

#include "OpResources.h"	// this is the only resources file with global resources


FltRangeCmd::FltRangeCmd ( const char *name, int active ) :  Cmd ( name, active )
{
	_popupRealised = False;
}


void FltRangeCmd::doCommand()
{
	if ( _popupRealised == True )
		return;

	char buf[256];
	char help[320];
		
	_edit = new LineEdit ( ( char * ) name(), TRUE,
									&FltRangeCmd::enterCallback,
									&FltRangeCmd::cancelCallback, ( void * ) this );	
                
	/* retrieve resource to be displayed */
 	snprintf ( buf, sizeof ( buf ), "%f,%f",mixwaveResources->minFltY(), mixwaveResources->maxFltY() );
	buf [ sizeof ( buf ) - 1 ] = 0;
	_edit->setColumns ( strlen ( buf ) );	// set width before instantiation

	snprintf ( help, sizeof ( help ), "Specifies floating-point value range for view: min,max; default %f,%f",
				mixwaveResources->minFltY(), mixwaveResources->maxFltY() );
				
	help [ sizeof ( help ) - 1 ] = 0;
	_edit->setHelp ( help );
	_edit->execute();
	_edit->setText ( buf );	// can't set text before instantiation...
	_popupRealised = True;
}


void FltRangeCmd::undoCommand()
{
	// not implemented
}


void FltRangeCmd::enterCallback ( void *clientData, char *text )
{
	FltRangeCmd *obj = ( FltRangeCmd * ) clientData;

	if ( strlen ( text ) > 0 )
		obj->enter ( text );

	obj->_popupRealised = False;
	delete obj->_edit;
}

/* specify resource to be set on 'enter' or 'ok' */
void FltRangeCmd::enter ( char *text )
{
	// check for separator
	int sep = 0;
	
	for ( unsigned int count = 0; count < strlen ( text ); count++ )
		if ( count[text] == ',' )
			sep++;

	if ( sep < 1 )
	{
		char msg[300];
		snprintf ( msg, sizeof ( msg ), "%s invalid: missing comma separator.\n\
Format (float): minYvalue,maxYvalue", text );
		msg [ sizeof ( msg ) -  1 ] = 0;
		infoDialogManager->post ( msg );
		return;
	}

	float minY, maxY;;

	sscanf ( text, "%f,%f", &minY, &maxY );

	if ( minY < maxY )
	{
		mixwaveResources->minFltY ( minY );
		mixwaveResources->maxFltY ( maxY );
	}
	else
	{
		mixwaveResources->minY ( -32768 );	// default to signed 16-bit
		mixwaveResources->maxY ( 32767 );
	}
}


void FltRangeCmd::cancelCallback ( void *clientData )
{
	FltRangeCmd *obj = ( FltRangeCmd * ) clientData;

	obj->_popupRealised = False;
	delete obj->_edit;
}
