/*
	NewMixCmd.cc

	Will 18, 2001: will@opaline.demon.co.uk
*/

#include <DMenuCmd.h>
#include "OpResources.h"
#include "NewMixCmd.h"
#include "AppWindow.h"
#include <MessageDialogManager.h>
#include <InfoDialogManager.h>
#include <TextEdit.h>
#include <LineEdit.h>


NewMixCmd::NewMixCmd ( char *name, int active ) : NoUndoCmd ( name, active )
{
	_menuRealised = False;
	_name = name;
	_waveNum = 0;
	_menuRealised = False;
	_editRealised = False;
	_panMenuRealised = False;
	_gainMenuRealised = False;
	_panEditRealised = False;
	_gainEditRealised = False;
	_waveNum = 0;
}


void NewMixCmd::doCommand()
{
	if ( _menuRealised == True )
		return;

	if ( wavePak->Count() == 0 )
		return;
		
	unsigned int n;
	unsigned int maxNameLength = 0;

	_arrayItems = wavePak->Count();

	for ( n = 0; n < _arrayItems; n++ )
		if ( strlen ( (*wavePak)[n].pWave->name() ) > maxNameLength )
			maxNameLength = strlen ( (*wavePak)[n].pWave->name() );

	n = 0;
	maxNameLength++;
	
	if ( maxNameLength < 5 )
		maxNameLength = 5;	// at least 'quit'

	_arrayItems++;
	_edit = new TextEdit ( ( char * ) _name, TRUE, &NewMixCmd::enterCallback,
							&NewMixCmd::cancelCallback, ( void * ) this );
	
	_editRealised = True;
	_edit->setRows ( ( ( _arrayItems - 1 ) * 4 ) + 1 );
	_edit->setColumns ( maxNameLength + 20 );
	_edit->execute();
		
	char waveStrings [ _arrayItems ] [ maxNameLength ];

	for ( n = 0; n < ( _arrayItems - 1 ); n++ )
		strcpy ( ( char * ) &waveStrings [ n ], (*wavePak)[n].pWave->name() );

	strcpy ( waveStrings [ n ], "Quit" );

	_waveMenu = new DMenuCmd ( "New mix: select wave", TRUE, FALSE,
								&NewMixCmd::waveMenuSelectionCallback,
								( void * ) this, "Current waves",
								_arrayItems, maxNameLength, ( char * ) &waveStrings );
	
	_menuRealised = True;
	_waveMenu->execute();

	maxNameLength = 0;
	_arrayItems = templatePak->Count();

	for ( n = 0; n < _arrayItems; n++ )
		if ( strlen ( (*templatePak)[n].name ) > maxNameLength )
			maxNameLength = strlen ( (*templatePak)[n].name );

	n = 0;
	maxNameLength++;
	
	if ( maxNameLength < 14 )
		maxNameLength = 14;	// accomodate 'quit' at a minimum

	_arrayItems += 2;	// 'Global value' and 'quit'
	char templateStrings [ _arrayItems ] [ maxNameLength ];

	for ( n = 0; n < ( _arrayItems - 2 ); n++ )
		strcpy ( ( char * ) &templateStrings [ n ], (*templatePak)[n].name );

	strcpy ( templateStrings [ n ], "Global value" ); n++;
	strcpy ( templateStrings [ n ], "Quit" );
	
	_panMenu = new DMenuCmd ( "PanMenu", TRUE, FALSE,
								&NewMixCmd::panMenuSelectionCallback,
								( void * ) this, "Wave: none    ",
								_arrayItems, maxNameLength, ( char * ) &templateStrings );

	_panMenuRealised = True;
	_panMenu->execute();
	
	_gainMenu = new DMenuCmd ( "GainMenu", TRUE, FALSE,
								&NewMixCmd::gainMenuSelectionCallback,
								( void * ) this, "Wave: none    ",
								_arrayItems, maxNameLength, ( char * ) &templateStrings );

	_gainMenuRealised = True;
	_gainMenu->execute();
	
	_panEdit = new LineEdit ( "Set pan", TRUE,
									&NewMixCmd::panEnteredCallback,
									&NewMixCmd::panCancelledCallback, ( void * ) this );

	_gainEdit = new LineEdit ( "Set gain", TRUE,
									&NewMixCmd::gainEnteredCallback,
									&NewMixCmd::gainCancelledCallback, ( void * ) this );

	_utils = new MixwaveUtils;
	showMixData();
}


void NewMixCmd::waveMenuSelectionCallback ( void *clientData, unsigned int selectedItemNum, char *label )
{
	NewMixCmd *obj = ( NewMixCmd * ) clientData;				

	int error = 0;
	// Since the menu is static with reference to the name strings
	// (quitOnSelect is false), the reference may be invalid; check by name.
	// also check if WM 'X' pressed

	if ( ( selectedItemNum == 0 ) || ( strcmp ( label, "Quit" ) == 0 ) )
	{
		obj->_waveNum = 0;
		
		if ( obj->_gainEditRealised == True )
			obj->_gainEdit->close();

		if ( obj->_panEditRealised == True )
			obj->_panEdit->close();
		
		if ( obj->_editRealised == True )
			obj->_edit->close();

		obj->_menuRealised = False;
		obj->_editRealised = False;

		delete obj->_utils;
		delete obj->_gainEdit;
		delete obj->_panEdit;
		delete obj->_gainMenu;		
		delete obj->_panMenu;
		delete obj->_waveMenu;
		delete obj->_edit;
	}
	else
	{
		int count, found;
	
		// scan base list for returned name pak index
		for ( count = 0; count < wavePak->Count(); count++ )
		{
			found = 0;

			if ( strcmp ( label, (*wavePak)[count].pWave->name() ) == 0 )
			{
				if ( (*wavePak)[count].fLock == ACTIVE )
				{
					if ( mixwaveResources->enableWarningPopups() == True )
					{
						char msg[BUFSIZE];
						snprintf ( msg, sizeof ( msg ), "%s\nFile in use!", (*wavePak)[count].pWave->name() );
						msg [ sizeof ( msg ) - 1 ] = 0;
						messageDialogManager->post ( msg );
					}
					
					error = 1;
				}
				else
				{
					obj->_waveNum = count + 1;
					obj->waveSelected();
					found = 1;
					count = wavePak->Count();
				}
			}
		}
		
		if ( error )
			return;

		if ( !found )
		{
			char msg[BUFSIZE];
			snprintf ( msg, sizeof ( msg ), "Error: %s\nFile not loaded.", label );
			msg [ sizeof ( msg ) - 1 ] = 0;
			messageDialogManager->post ( msg );
		}
	}
}


void NewMixCmd::waveSelected()
{
	char title [ strlen ( _utils->fileFromPath ( (*wavePak)[_waveNum-1].pWave->name() ) ) + 1 ];

	if ( _panMenuRealised == True )
	{
		strncpy ( title, _utils->fileFromPath ( (*wavePak)[_waveNum-1].pWave->name() ), sizeof ( title ) );
		title [ strlen ( title ) ] = 0;
		_panMenu->setTitle ( title );
	}
	
	if ( _gainMenuRealised == True )
	{
		strncpy ( title, _utils->fileFromPath ( (*wavePak)[_waveNum-1].pWave->name() ), sizeof ( title ) );
		title [ strlen ( title ) ] = 0;
		_gainMenu->setTitle ( title );
	}
}


void NewMixCmd::panMenuSelectionCallback ( void *clientData, unsigned int selectedItemNum, char *label )
{
	NewMixCmd *obj = ( NewMixCmd * ) clientData;

	if ( ( selectedItemNum == obj->_arrayItems ) || ( selectedItemNum == 0 ) ) // WM 'X' or 'quit menu' selected
	{
		obj->_panMenuRealised = False;
		obj->_waveNum = 0;
	}
	else
	{
		if ( obj->_waveNum != 0 )
		{
			if ( selectedItemNum == ( obj->_arrayItems - 1 ) ) 
				obj->setPan();
			else
				obj->panSelected ( label );
		}
	}
}


void NewMixCmd::panSelected ( char *label )
{
	int error = 0;
	char temp[strlen ( _utils->fileFromPath ( label ) ) + 1];
	char msg[BUFSIZE];

	strcpy ( temp, _utils->fileFromPath ( label ) );

// suffix s/b defined in resources; also need to set filter in browser
	if ( strstr ( temp, ".flt" ) == NULL )
	{
		snprintf ( msg, sizeof ( msg ), "%s\nNot floating point - expected '.flt' suffix.", label );
		msg [ sizeof ( msg ) - 1 ] = 0;			
		messageDialogManager->post ( msg );
		return;
	}

	int count, found, n;
	
	// scan base list for returned name pak index
	for ( count = 0; count < templatePak->Count(); count++ )
	{
		found = 0;

		if ( strcmp ( label, (*templatePak)[count].name ) == 0 )
		{
			if ( (*templatePak)[count].dataSize == 0 )
			{
				if ( mixwaveResources->enableWarningPopups() == True )
				{
					snprintf ( msg, sizeof ( msg ), "Error: %s\nFile is empty.", label );
					msg [ sizeof ( msg ) - 1 ] = 0;
					messageDialogManager->post ( msg );
				}

				error = 1;
			}

			for ( n = 0; n < wavePak->Count(); n++)
			{
				if ( (*templatePak)[(*wavePak)[n].panTemplate-1].templateNum == ( count + 1 )
						|| (*templatePak)[(*wavePak)[n].gainTemplate-1].templateNum == ( count + 1 ) )
				{	// template with this num has already been allocated to a wave
					if ( mixwaveResources->enableWarningPopups() == True )
					{
						snprintf ( msg, sizeof ( msg ), "%s\nTemplate already associated with\n%s", label, (*wavePak)[n].pWave->name() );
						msg [ sizeof ( msg ) - 1 ] = 0;
						messageDialogManager->post ( msg );
					}

					error = 1;
					n = wavePak->Count();
				}
			}

			if ( error )
				return;

			(*wavePak)[_waveNum-1].panTemplate = count + 1;
			(*wavePak)[_waveNum-1].panRatio = ( float ) (*wavePak)[_waveNum-1].pWave->totalSamples() / ( float ) (*templatePak)[count].dataSize;
			found = 1;
			count = templatePak->Count();
		}
	}

	if ( !found )
	{
		snprintf ( msg, sizeof ( msg ), "Error: %s\nFile not loaded.", label );
		msg [ sizeof ( msg ) - 1 ] = 0;			
		messageDialogManager->post ( msg );
	}

	if ( _editRealised == True )
		showMixData();
}


void NewMixCmd::setPan()
{
	if ( _panEditRealised == True )
		return;

	char buf[256];
	char help[256];
		
	/* retrieve resource to be displayed */
 	snprintf ( buf, sizeof ( buf ), "%f", (*wavePak)[_waveNum-1].pan );

	buf [ sizeof ( buf ) - 1 ] = 0;		
	_panEdit->setColumns ( strlen ( buf ) );	// set width before instantiation

	snprintf ( help, sizeof ( help ), "Specifies pan: currently %f", (*wavePak)[_waveNum-1].pan);				
	help [ sizeof ( help ) - 1 ] = 0;
	_panEdit->setHelp ( help );
	_panEditRealised = True;
	_panEdit->execute();
	_panEdit->setText ( buf );	// can't set text before instantiation...
}


void NewMixCmd::gainMenuSelectionCallback ( void *clientData, unsigned int selectedItemNum, char *label )
{
	NewMixCmd *obj = ( NewMixCmd * ) clientData;

	if ( ( selectedItemNum == obj->_arrayItems ) || ( selectedItemNum == 0 ) )
	{
		obj->_gainMenuRealised = False;
		obj->_waveNum = 0;
	}
	else
	{
		if ( obj->_waveNum != 0 )
		{
			if ( selectedItemNum == ( obj->_arrayItems - 1 ) ) 
				obj->setGain();
			else
				obj->gainSelected ( label );
		}
	}
}


void NewMixCmd::gainSelected ( char *label )
{
	int error = 0;
	char temp[strlen ( _utils->fileFromPath ( label ) ) + 1];
	char msg[BUFSIZE];

	strcpy ( temp, _utils->fileFromPath ( label ) );

	if ( strstr ( temp, ".flt" ) == NULL )
	{
		char msg[BUFSIZE];
	
		snprintf ( msg, sizeof ( msg ), "%s\nNot floating point - expected '.flt' suffix.", label );
		msg [ sizeof ( msg ) - 1 ] = 0;			
		messageDialogManager->post ( msg );
		return;
	}

	int count, found, n;
	
	// scan base list for returned name pak index
	for ( count = 0; count < templatePak->Count(); count++ )
	{
		found = 0;

		if ( strcmp ( label, (*templatePak)[count].name ) == 0 )
		{
			if ( (*templatePak)[count].dataSize == 0 )
			{
				if ( mixwaveResources->enableWarningPopups() == True )
				{
					snprintf ( msg, sizeof ( msg ), "Error: %s\nFile is empty.", label );
					msg [ sizeof ( msg ) - 1 ] = 0;
					messageDialogManager->post ( msg );
				}

				error = 1;
			}

			for ( n = 0; n < wavePak->Count(); n++)
			{
				if ( (*templatePak)[(*wavePak)[n].gainTemplate-1].templateNum == ( count + 1 ) 
						|| (*templatePak)[(*wavePak)[n].panTemplate-1].templateNum == ( count + 1 ) )
				{	// template with this num has already been allocated to a wave
					if ( mixwaveResources->enableWarningPopups() == True )
					{
						snprintf ( msg, sizeof ( msg ), "%s\nTemplate already associated with\n%s", label, (*wavePak)[n].pWave->name() );
						msg [ sizeof ( msg ) - 1 ] = 0;
						messageDialogManager->post ( msg );
					}

					error = 1;
					n = wavePak->Count();
				}
			}

			if ( error )
				return;

			(*wavePak)[_waveNum-1].gainTemplate = count + 1;
			(*wavePak)[_waveNum-1].gainRatio = ( float ) (*wavePak)[_waveNum-1].pWave->totalSamples() / ( float ) (*templatePak)[count].dataSize;
			found = 1;
			count = templatePak->Count();
		}
	}

	if ( !found )
	{
		char msg[BUFSIZE];
		snprintf ( msg, sizeof ( msg ), "Error: %s\nFile not loaded.", label );
		msg [ sizeof ( msg ) - 1 ] = 0;			
		messageDialogManager->post ( msg );
	}

	if ( _editRealised == True )
		showMixData();
}


void NewMixCmd::setGain()
{
	if ( _gainEditRealised == True )
		return;
		
	char buf[256];
	char help[256];

	/* retrieve resource to be displayed */
 	snprintf ( buf, sizeof ( buf ), "%f", (*wavePak)[_waveNum-1].gain );

	buf [ sizeof ( buf ) - 1 ] = 0;		
	_gainEdit->setColumns ( strlen ( buf ) );	// set width before instantiation

	snprintf ( help, sizeof ( help ), "Specifies gain: currently %f", (*wavePak)[_waveNum-1].gain );				
	help [ sizeof ( help ) - 1 ] = 0;
	_gainEdit->setHelp ( help );
	_gainEditRealised = True;
	_gainEdit->execute();
	_gainEdit->setText ( buf );	// can't set text before instantiation...
}


void NewMixCmd::showMixData()
{
	int count, length;
	
	length = 0;
	
	for ( count = 0; count < wavePak->Count(); count++ )
	{
		length += strlen ( (*wavePak)[count].pWave->name() );
		
		if ( (*wavePak)[count].panTemplate != 0 )
			length += strlen ( (*templatePak)[(*wavePak)[count].panTemplate-1].name );

		if ( (*wavePak)[count].gainTemplate != 0 )
			length += strlen ( (*templatePak)[(*wavePak)[count].gainTemplate-1].name );

		length += ( 68 * wavePak->Count() );
	}

	char buf[length];
	buf[0] = 0;

	for ( count = 0; count < wavePak->Count(); count++ )
	{
		strcat ( buf, "Wave: " );
		strncat ( buf, (*wavePak)[count].pWave->name(), sizeof ( buf ) - strlen ( buf ) - 1 );
		buf [ sizeof ( buf ) - 1 ] = 0;
		strncat ( buf, "\nPan template: ", sizeof ( buf ) - strlen ( buf ) - 1 );
		buf [ sizeof ( buf ) - 1 ] = 0;

		if ( (*wavePak)[count].panTemplate != 0 )
		{
			strncat ( buf, (*templatePak)[(*wavePak)[count].panTemplate-1].name, sizeof ( buf ) - strlen ( buf ) - 1 );
			buf [ sizeof ( buf ) - 1 ] = 0;
		}
		else
		{
			char num[20];
			snprintf ( num, sizeof ( num ), "global %f", (*wavePak)[count].pan );
			num [ sizeof ( num ) - 1 ] = 0;
			strncat ( buf, num, sizeof ( buf ) - strlen ( buf ) - 1 );
			buf [ sizeof ( buf ) - 1 ] = 0;
		}
			 
		strncat ( buf, "\nGain template: ", sizeof ( buf ) - strlen ( buf ) - 1 );
		buf [ sizeof ( buf ) - 1 ] = 0;
	
		if ( (*wavePak)[count].gainTemplate != 0 )
		{
			strncat ( buf, (*templatePak)[(*wavePak)[count].gainTemplate-1].name, sizeof ( buf ) - strlen ( buf ) - 1 );
			buf [ sizeof ( buf ) - 1 ] = 0;
		}
		else
		{
			char num[20];
			snprintf ( num, sizeof ( num ), "global %f", (*wavePak)[count].gain );
			num [ sizeof ( num ) - 1 ] = 0;
			strncat ( buf, num, sizeof ( buf ) - strlen ( buf ) - 1 );
			buf [ sizeof ( buf ) - 1 ] = 0;
		}

		strncat ( buf, "\n\n", sizeof ( buf ) - strlen ( buf ) - 1 );
		buf [ sizeof ( buf ) - 1 ] = 0;
	}

	_edit->setText ( buf );
}


void NewMixCmd::enterCallback ( void *clientData, char *text )
{
	// no action required (but could parse text if editing
	// allowed on object creation... _edit->setStatus ( True ) )
	NewMixCmd *obj = ( NewMixCmd * ) clientData;

	obj->_editRealised = False;
}


void NewMixCmd::cancelCallback ( void *clientData )
{
	NewMixCmd *obj = ( NewMixCmd * ) clientData;

	obj->_editRealised = False;	// called by WM 'X' in TextEdit
}


void NewMixCmd::panEnteredCallback ( void *clientData, char *text )
{
	NewMixCmd *obj = ( NewMixCmd * ) clientData;

	obj->setMix ( text, 0 );	// '0' = pan, '1' = gain
	obj->_panEditRealised = False;
}


void NewMixCmd::panCancelledCallback ( void *clientData )
{
	NewMixCmd *obj = ( NewMixCmd * ) clientData;

	obj->_panEditRealised = False;
}


void NewMixCmd::gainEnteredCallback ( void *clientData, char *text )
{
	NewMixCmd *obj = ( NewMixCmd * ) clientData;

	obj->setMix ( text, 1 );
	obj->_gainEditRealised = False;	
}


void NewMixCmd::gainCancelledCallback ( void *clientData )
{
	NewMixCmd *obj = ( NewMixCmd * ) clientData;

	obj->_gainEditRealised = False;
}


void NewMixCmd::setMix ( char *text, int paramType )
{
	// 0 = pan, 1 = gain
	char msg[BUFSIZE];
	float value;
	
	if ( strlen ( text ) == 0 )
	{
		strncpy ( msg, "No parameter value entered!", sizeof ( msg ) );
		msg [ sizeof ( msg ) - 1 ] = 0;
		messageDialogManager->post ( msg );
		return;
	}
	else
		sscanf ( text, "%f", &value );
	
	switch ( paramType )
	{
		case 0:	// pan
			if ( ( value < 0 ) || ( value > 1 ) )
			{
				snprintf ( msg, sizeof ( msg ), "Invalid pan %f - should be in range 0 to 1.", value );
				msg [ sizeof ( msg ) - 1 ] = 0;
				messageDialogManager->post ( msg );
				return;
			}
			else
			{
				(*wavePak)[_waveNum-1].pan = value;
				(*wavePak)[_waveNum-1].panTemplate = 0;
			}
			
			break;
		
		case 1:	// gain
			if ( value < 0 )
			{
				snprintf ( msg, sizeof ( msg ), "Invalid gain %f - should be a positive factor.", value );
				msg [ sizeof ( msg ) - 1 ] = 0;
				messageDialogManager->post ( msg );
				return;
			}
			else
			{
				(*wavePak)[_waveNum-1].gain = value;
				(*wavePak)[_waveNum-1].gainTemplate = 0;
			}
			
			break;
	}

	if ( _editRealised == True )
		showMixData();
}
