/*
	OpResources.cc

	Will Eighteen, 2000: will@opaline.demon.co.uk
*/

#include "OpResources.h"
#include <stdlib.h>
#include <sys/stat.h>

#define BUFSIZE 1024


OpResources::OpResources ( char *paramsFileName )
{
	struct stat statInfo;
	char file[256];
	char *opResFile;
	
	_error[0] = 0;
	_appResources.saveOnExit = 0;

	strncpy ( _fileName, paramsFileName, 63 );
	strcpy ( file, ( char * ) getenv("HOME"));
	strcat ( file, "/" );
	strncpy ( _appResources.workingDir, file, sizeof ( _appResources.workingDir ) );
	strcat ( file, ( char * ) paramsFileName );
	opResFile = ( char * ) &file;
	strncpy ( _opResourceFile, opResFile, 255 ); /*store the resource file name */

	if ( ( _file = fopen ( opResFile, "r" ) ) == NULL )
	{
		printf("creating new resource file %s in your home directory.", _fileName);
		initOpResources();
		setOpResources();
	}
	else
	{
		if ( stat ( opResFile, &statInfo ) == 0 )
			_fileSize = statInfo.st_size;

		if ( _fileSize == 0 )
		{
			printf ( _error, "%s is empty!", opResFile );
			fclose ( _file );
			opResFile = 0;
		}
		else
			getOpResources();
	}	
}


OpResources::~OpResources()
{
	if ( _appResources.saveOnExit )
		setOpResources();
}


void OpResources::initOpResources()
{
	_appResources.saveOnExit = 1;	// true

 	/* user-definable structure contents */
	_appResources.blockLength = 100000;	// bytes
	_appResources.bitsPerSample = 16;
	_appResources.samplesPerSec = 44100;
	_appResources.channels = 1;
	_appResources.numDigits = 10;	// default length of text display boxes
	_appResources.fpDigits = 5;		// single-sample resolution at 44.1 kHz
	strncpy ( _appResources.viewWindowSize, "270x436", sizeof ( _appResources. viewWindowSize ) );
	_appResources.CWF_callbacktimeout = 500;
	_appResources.defaultWavNumSamples = 0;	// default zero-value samples in new .wav
	_appResources.minY = -32768;
	_appResources.maxY = 32767;
	_appResources.minFltY = -1.0;
	_appResources.maxFltY = 1.0;
	_appResources.enableMessagePopups = True;
	_appResources.enableWarningPopups = True;
	_appResources.convertOffsetInt = 0;
	_appResources.convertScaleInt = 1;
	_appResources.convertOffsetFloat = 0;
	_appResources.convertScaleFloat = 1.0;
	_appResources.defaultTemplateSize = 1000;
	_appResources.linkViews = False;
	_appResources.syncTemplateLength = False;
	_appResources.syncSelections = False;
	strncpy ( _appResources.playCmd, "play", sizeof ( _appResources.playCmd ) );
	strncpy ( _appResources.playPreamble, " '", sizeof ( _appResources.playPreamble ) );
	strncpy ( _appResources.playPostamble, "' 2> ", sizeof ( _appResources.playPostamble ) );
	strncpy ( _appResources.playPostfix, " &", sizeof ( _appResources.playPostfix ) );
	strncpy ( _appResources.moveCmd, "/bin/mv", sizeof ( _appResources.moveCmd ) );
	strncpy ( _appResources.removeCmd, "/bin/rm", sizeof ( _appResources.removeCmd ) );
	strncpy ( _appResources.pidofCmd, "/sbin/pidof", sizeof ( _appResources.pidofCmd ) );
	strncpy ( _appResources.killCmd, "/bin/kill", sizeof ( _appResources.killCmd ) );
	strncpy ( _appResources.shEvalPrefix, " $(", sizeof ( _appResources.shEvalPrefix ) );
	strncpy ( _appResources.shEvalSuffix, ")", sizeof ( _appResources.shEvalSuffix ) );
	_appResources.currentWave[0] = 0;
	_appResources.gain = 1;		// default unity gain, centre pan
	_appResources.pan = 0.5;	// for 'new wave'
	_appResources.enableRemix = True;
	_appResources.mixToCurrent = False;

	// object status
	_appResources.waveCtrlRealised = False;
	_appResources.mixing = False;

	// csound extras
	_appResources.csoundCmd[0] = 0;
	strncpy ( _appResources.cspatialScoreSuffix, "_spatial.sco", sizeof ( _appResources.cspatialScoreSuffix ) );
	strncpy ( _appResources.cspatialOrchSuffix, "_spatial.orc", sizeof ( _appResources.cspatialScoreSuffix ) );
	// output a file of type 'raw'; spatialisation routine calls RawToWavCmd
	strncpy ( _appResources.csoundRunFlags, "-d -h -s -o", sizeof ( _appResources.csoundRunFlags ) );
	strncpy ( _appResources.csoundPidlogFile, "csoundPid.log", sizeof ( _appResources.csoundPidlogFile ) );
	strncpy ( _appResources.csoundRunlogFile, "csoundRun.log", sizeof ( _appResources.csoundRunlogFile ) );
	strncpy ( _appResources.csoundRunError, "error:", sizeof ( _appResources.csoundRunError ) );
	_appResources.csoundSampleSize = 10;	// automatically determined if specified zero
	_appResources.kRate = 4410;
	_appResources.maxGenData = 16777216;	// csound-imposed limit of 2^24 data points
}


void OpResources::setOpResources()
{
	if ( ( _file = fopen ( _opResourceFile, "w" ) ) == NULL )
		printf ( "Can't write resources %s", _opResourceFile );
	else
	{
		fwrite ( &_appResources, sizeof ( _appResources ), 1, _file );
		fclose ( _file );
	}
}


void OpResources::getOpResources()
{
	if ( ( _file = fopen ( _opResourceFile, "r" ) ) == NULL )
		sprintf ( _error, "Can't read resources %s", _opResourceFile );
	else
	{
		fread ( &_appResources, sizeof ( _appResources ), 1, _file );
		fclose ( _file );
	}
}


void OpResources::setSaveOnExit ( int status )
{
	if ( status != 0 )
		status = 1;

	_appResources.saveOnExit = status;
	setOpResources();	// or the flag isn't saved...
}


void OpResources::save()
{
    setOpResources();	// save immediate if resources are changed at runtime
}
