/*
 CmdInterface.h - extend Interface class to support Cmd interaction

 Douglas A. Young, 'Object-oriented Programming with C++ and OSF/Motif'
 2nd ed., Prentice-Hall.
*/

#ifndef CMDINTERFACE_H
#define CMDINTERFACE_H

#include "Interface.h"

class Cmd;

class CmdInterface : public Interface
{
	friend class Cmd;
	
	protected:
		Cmd *_cmd;
		static void executeCmdCallback ( Widget, XtPointer, XtPointer );
		int _active;
		CmdInterface ( Cmd * );
		virtual void activate();
		virtual void deactivate();
};
	
#endif

/*
  Notes:
  
  This class accepts (de)activate() messages from Cmd objects, changing the
  sensitivity of the component's base widget to match the state of the associated
  Cmd object
  
  Each CmdInterface object is automatically registered with a Cmd object,
  which must be specified in the CmdInterface constructor.
  
  The CmdInterface class supports a callback function which can be used to
  execute the associated Cmd object. Since this class does not create a
  widget, it cannot register the callback; derived classes register this callback
  function with an appropriate widget.
  
  Note that each CmdInterface object is associated with a single Cmd
  object, although a Cmd object may be associated with multiple
  CmdInterface objects.
*/
